<?php
App::uses('EvInventoryAppModel', 'EvInventory.Model');
App::uses('InventoryActions', 'EvInventory.Lib');

/**
 * EvInventoryInventory Model
 *
 */
class Inventory extends EvInventoryAppModel {

	/**
	 * constructor - setup the dynamic relationships
	 *
	 */
	public function __construct($id = false, $table = null, $ds = null) {
		parent::__construct($id, $table, $ds);

		$relationships = Configure::read('EvInventory.InventoryBelongsTo');
		if (! empty($relationships)) {
			$belongsTo = array();
			foreach ($relationships as $key => $value) {
				$belongsTo[$key] = $value;
			}

			if (! empty($belongsTo)) {
				$this->bindModel(
					array(
						'belongsTo' => $belongsTo
					),
					false
				);
			}
		}
	}

	/**
	 * given the model name and the id get it's inventory
	 *
	 * @todo 	Refactor out at some point for better testable methods
	 * @param 	string 		$modelName 	The model name it belongs to
	 * @param 	int 		$modelId 	The models item id
	 * @return 	array|bool 	$inventory 	bool false on fail or inventory array
	 */
	public function getInventory($modelName, $modelId) {
		$inventoryContain = Configure::read('EvInventory.InventoryContains');

		if (empty($inventoryContain)) {
			$inventoryContain = array();

			$relationships = Configure::read('EvInventory.InventoryBelongsTo');

			foreach ($relationships as $key => $value) {
				if (is_array($value)) {
					$inventoryContain[] = $key;
				} else {
					$inventoryContain[] = $value;
				}
			}
		}

		$inventory = $this->find(
			'first',
			array(
				'conditions' => array(
					'Inventory.model' => $modelName,
					'Inventory.model_id' => $modelId
				),
				'contain' => $inventoryContain
			)
		);

		if (empty($inventory)) {
			return false;
		}

		return $inventory;
	}

	/**
	 * warning level reached, process the warning levl actions
	 *
	 * @param 	array 	$inventory 	The inventory row
	 * @return 	bool
	 */
	public function warningLevelReached($inventory) {
		if (! is_array($inventory['Inventory']['warning_action'])) {
			return true;
		}

		$ActionsLib = new InventoryActions($inventory, 'warning');

		foreach ($inventory['Inventory']['warning_action'] as $action) {
			if (method_exists($ActionsLib, $action)) {
				$ActionsLib->$action();
			}
		}

		return true;
	}

	/**
	 * out of stock, process the out of stock actions
	 *
	 * @param 	array 	$inventory 	The inventory row
	 * @return 	bool
	 */
	public function outOfStock($inventory) {
		if (! is_array($inventory['Inventory']['oos_action'])) {
			return true;
		}

		$ActionsLib = new InventoryActions($inventory, 'outofstock');

		foreach ($inventory['Inventory']['oos_action'] as $action) {
			if (method_exists($ActionsLib, $action)) {
				$ActionsLib->$action();
			}
		}

		return true;
	}

/**
 * JSON encode the arrays
 * @param  [type] $data    [description]
 * @param  array  $options [description]
 * @return [type]          [description]
 */
	public function beforeSave($options = array()) {
		if (! empty($this->data[$this->alias]['warning_action']) && is_array($this->data[$this->alias]['warning_action'])) {
			$this->data[$this->alias]['warning_action'] = json_encode($this->data[$this->alias]['warning_action']);
		}
		if (! empty($this->data[$this->alias]['oos_action']) && is_array($this->data[$this->alias]['oos_action'])) {
			$this->data[$this->alias]['oos_action'] = json_encode($this->data[$this->alias]['oos_action']);
		}
	}

/**
 * After find callback. Can be used to modify any results returned by find.
 *
 * @param mixed $results The results of the find operation
 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
 * @return mixed An array value will replace the value of $results - any other value will be ignored.
 */
	public function afterFind($results, $primary = false) {
		foreach ($results as $key => $item) {
			if (! empty($item[$this->alias]['warning_action'])) {
				$results[$key][$this->alias]['warning_action'] = json_decode($item[$this->alias]['warning_action'], true);
			} else {
				$results[$key][$this->alias]['warning_action'] = [];
			}

			if (! empty($item[$this->alias]['oos_action'])) {
				$results[$key][$this->alias]['oos_action'] = json_decode($item[$this->alias]['oos_action'], true);
			} else {
				$results[$key][$this->alias]['oos_action'] = [];
			}
		}

		return $results;
	}
}
